!arno start
!  ________________________   EXTERNAL FILE

      ! Initialises SPHysics based on a IPART_B file which is exported by a blender script
      ! and further userdefined variables that describe amongst other the movement behaviour
      ! of the moving boundary particles
     
      subroutine external_file(g,expont,dx,dy,dz)
         use mv_shared
         use mv_vars
         use mstack
         include 'common.gen3D'
       
         integer mult(3)
         integer, allocatable :: p_tmpi(:,:,:) ! particle buffer for filling
         integer ico(3) !array indices
         integer aout(1)
         double precision rout(1)
         double precision t, x,y,z ! positions
         real nx, ny, nz ! normals
         real seed(3) !seed points for filling
         integer nseed !number of seeds
         real mat,mat_o ! material, old material
         character(20) mv_file
         integer xp1,xm1,yp1,ym1,zp1,zm1,b_all,tmp,filled !bits
         integer time(8)
         integer mati
         real min_fluid, max_fluid ! for determining fluid height
         type(rstack) mats
         
         print*
         print*,'--- Generating Geometry from IPART_B ---'
         print*
         
         ! Read in IPART_B
         open(131,file='IPART_B')
         read(131,*) dx
         read(131,*) dy
         read(131,*) dz
         write(*,*) ' Spacing dx,dy,dz ' ! was already defined in blender
         write(*,*) dx, dy, dz
         read(131,*) np
         write(*,*) ' Initial particle count '
         write(*,*) np
         read(131,*) mult(1)
         read(131,*) mult(2)
         read(131,*) mult(3)
         write(*,*) ' 3D Array size '
         write(*,*) mult(1), mult(2), mult(3)
         
         vlx = (mult(1)+0.5*iBC)*dx
         vly = (mult(2)+0.5*iBC)*dy
         vlz = (mult(3)+0.5*iBC)*dz
         
         allocate(p_tmp(mult(1),mult(2),mult(3),1))!a-note: this 4 dim vector at the start could be 3d but if we want normals later on we would need it 4d
         allocate(p_tmpi(mult(1),mult(2),mult(3))) 
         fx = -1
         do i=1,mult(1)
            do j=1,mult(2)
               do k=1,mult(3)
                  p_tmp(i,j,k,:) = (/ -1/)
                  p_tmpi(i,j,k) = 0
               end do
            end do
         end do
         nbf = -1
         nb = -1
         nbfm = -1
         nn = 0
         f_zmax = 0
         mat_o = 2
         call init(fbi,1)
         call init(mbi,1)
         call init(f_mv,4)
         call init(mats,1)
         do i=1,np
            read(131,*) x, y, z, mat, nx, ny, nz
            !a-todo: normals
            ico(1) = floor(x/dx+0.01)+1
            ico(2) = floor(y/dy+0.01)+1
            ico(3) = floor(z/dz+0.01)+1
            p_tmp(ico(1),ico(2),ico(3),1) = mat
            call push_single(mats,(/real(mat,kind(0.0d0))/))
         end do

         ! Start filling algorithm
         call init(fill, 3)
         seed = (/0.,0.,0./)
134      write(*,*) ' Please provide seed point for filling (x,y,z) '
         read(*,*) seed(1),seed(2),seed(3)
         write(*,*)seed(1),seed(2),seed(3)
         seed(1) = floor(seed(1)/dx+0.01)+1
         seed(2) = floor(seed(2)/dy+0.01)+1
         seed(3) = floor(seed(3)/dz+0.01)+1
         if(p_tmp(int(seed(1)),int(seed(2)),int(seed(3)),1).gt.0)then
            write(*,*) ' Invalid Seed point ', seed
            goto 134
         end if
         call push(fill,int(seed))
         write(*,*) ' Another seed point? (1 = yes, 0 = no) '
         read(*,*) seed(1)
         write(*,*) seed(1)
         if(seed(1).gt.0.5) goto 134
         nseed = get_len(fill)
         
         ! Init Universal Filling Algorithm
         xp1 = 1 !actually constants
         xm1 = 2
         yp1 = 4
         ym1 = 8
         zp1 = 16
         zm1 = 32
         b_all = 63
         filled = 64
         do i=1,nseed
            ico = get_el(fill, i)
            !go up until set particle is found
137         if(p_tmp(ico(1),ico(2),ico(3),1).ge.0)then
               call rep_el(fill, i, (/ ico(1), ico(2), ico(3)-1 /))
               p_tmpi(ico(1),ico(2),ico(3)) = 1
            else
               ico(3) = ico(3)+1
               goto 137
            end if
         end do

         ! Look at same height for other possibilities to go down
         i = 1
138      if(i.le.get_len(fill))then
            ico = get_el(fill,i)
            !go up until set particle is found
140         if(p_tmp(ico(1),ico(2),ico(3),1).ge.0)then
               call rep_el(fill, i, (/ ico(1), ico(2), ico(3)-1 /))
               p_tmpi(ico(1),ico(2),ico(3)) = 1
            else
               ico(3) = ico(3)+1
               goto 140
            end if
            ico(3) = ico(3)-1
            tmp = p_tmpi(ico(1),ico(2),ico(3))
            tmp = ior(tmp,zp1)
            if(p_tmp(ico(1)+1,ico(2),ico(3),1).lt.0.and.
     &         iand(tmp,xp1).eq.0)then
               call push(fill,(/ico(1)+1,ico(2),ico(3)/))
               p_tmpi(ico(1)+1,ico(2),ico(3)) = 
     &               ior(p_tmpi(ico(1)+1,ico(2),ico(3)),xm1)
               tmp = ior(tmp,xp1)
            end if
            if(p_tmp(ico(1)-1,ico(2),ico(3),1).lt.0.and.
     &         iand(tmp,xm1).eq.0)then
               call push(fill,(/ico(1)-1,ico(2),ico(3)/))
               p_tmpi(ico(1)-1,ico(2),ico(3)) =
     &               ior(p_tmpi(ico(1)-1,ico(2),ico(3)),xp1)
               tmp = ior(tmp,xm1)
            end if
            if(p_tmp(ico(1),ico(2)+1,ico(3),1).lt.0.and.
     &         iand(tmp,yp1).eq.0)then
               call push(fill,(/ico(1),ico(2)+1,ico(3)/))
               p_tmpi(ico(1),ico(2)+1,ico(3)) =
     &               ior(p_tmpi(ico(1),ico(2)+1,ico(3)),ym1)
               tmp = ior(tmp,yp1)
            end if
            if(p_tmp(ico(1),ico(2)-1,ico(3),1).lt.0.and.
     &         iand(tmp,ym1).eq.0)then
               call push(fill,(/ico(1),ico(2)-1,ico(3)/))
               p_tmpi(ico(1),ico(2)-1,ico(3)) =
     &               ior(p_tmpi(ico(1),ico(2)-1,ico(3)),yp1)
               tmp = ior(tmp,ym1)
            end if    
            p_tmpi(ico(1),ico(2),ico(3)) = tmp
            i = i+1
            goto 138
         end if
         !go down

152      do i = 1,get_len(fill)
            ico = get_el(fill,i)
            ico(3) = ico(3) - 1
141         if(p_tmp(ico(1),ico(2),ico(3),1).lt.0)then
               p_tmpi(ico(1),ico(2),ico(3)) = b_all
               p_tmpi(ico(1)+1,ico(2),ico(3)) =
     &               ior(p_tmpi(ico(1)+1,ico(2),ico(3)),xm1)
               p_tmpi(ico(1)-1,ico(2),ico(3)) =
     &               ior(p_tmpi(ico(1)-1,ico(2),ico(3)),xp1)
               p_tmpi(ico(1),ico(2)+1,ico(3)) =
     &               ior(p_tmpi(ico(1),ico(2)+1,ico(3)),ym1)
               p_tmpi(ico(1),ico(2)-1,ico(3)) =
     &               ior(p_tmpi(ico(1),ico(2)-1,ico(3)),yp1)
               p_tmpi(ico(1),ico(2),ico(3)+1) =
     &               ior(p_tmpi(ico(1),ico(2),ico(3)+1),zm1)
               p_tmpi(ico(1),ico(2),ico(3)-1) =
     &               ior(p_tmpi(ico(1),ico(2),ico(3)-1),zp1)
               ico(3) = ico(3) -1
               goto 141
            end if
         end do
         !empty fill
         call clear(fill)
         !add all not 333 and not 0 -1's particles to fill and start the old filling madness a little bit adapted

         do i=1,mult(1)
            do j=1,mult(2)
               do k=2,mult(3)
                  if(p_tmp(i,j,k,1).eq.-1.and.
     &               (p_tmpi(i,j,k).ne.0.and.
     &               p_tmpi(i,j,k).ne.b_all))then
                     call push(fill,(/i,j,k/))
                  end if
               end do
            end do
         end do
      
142      if(.not.empty(fill))then
            ico = get_el(fill,get_len(fill))
            call pop(fill)
            tmp = p_tmpi(ico(1),ico(2),ico(3))
            if(iand(tmp,xp1).eq.0)then
               p_tmpi(ico(1)+1,ico(2),ico(3)) =
     &               ior(p_tmpi(ico(1)+1,ico(2),ico(3)),xm1)
               if(p_tmpi(ico(1)+1,ico(2),ico(3)).ne.b_all.and.
     &            p_tmp(ico(1)+1,ico(2),ico(3),1).eq.-1)then
                  call push(fill,(/ico(1)+1,ico(2),ico(3)/))
               end if
            end if
            if(iand(tmp,xm1).eq.0)then
               p_tmpi(ico(1)-1,ico(2),ico(3)) =
     &               ior(p_tmpi(ico(1)-1,ico(2),ico(3)),xp1)
               if(p_tmpi(ico(1)-1,ico(2),ico(3)).ne.b_all.and.
     &            p_tmp(ico(1)-1,ico(2),ico(3),1).eq.-1)then
                  call push(fill,(/ico(1)-1,ico(2),ico(3)/))
               end if
            end if
            if(iand(tmp,yp1).eq.0)then
               p_tmpi(ico(1),ico(2)+1,ico(3)) =
     &               ior(p_tmpi(ico(1),ico(2)+1,ico(3)),ym1)
               if(p_tmpi(ico(1),ico(2)+1,ico(3)).ne.b_all.and.
     &            p_tmp(ico(1),ico(2)+1,ico(3),1).eq.-1)then
                  call push(fill,(/ico(1),ico(2)+1,ico(3)/))
               end if
            end if
            if(iand(tmp,ym1).eq.0)then
               p_tmpi(ico(1),ico(2)-1,ico(3)) =
     &               ior(p_tmpi(ico(1),ico(2)-1,ico(3)),yp1)
               if(p_tmpi(ico(1),ico(2)-1,ico(3)).ne.b_all.and.
     &            p_tmp(ico(1),ico(2)-1,ico(3),1).eq.-1)then
                  call push(fill,(/ico(1),ico(2)-1,ico(3)/))
               end if
            end if
            if(iand(tmp,zp1).eq.0)then
               p_tmpi(ico(1),ico(2),ico(3)+1) =
     &               ior(p_tmpi(ico(1),ico(2),ico(3)+1),zm1)
               if(p_tmpi(ico(1),ico(2),ico(3)+1).ne.b_all.and.
     &            p_tmp(ico(1),ico(2),ico(3)+1,1).eq.-1)then
                  call push(fill,(/ico(1),ico(2),ico(3)+1/))
               end if
            end if
            if(iand(tmp,zm1).eq.0)then
               p_tmpi(ico(1),ico(2),ico(3)-1) =
     &               ior(p_tmpi(ico(1),ico(2),ico(3)-1),zp1)
               if(p_tmpi(ico(1),ico(2),ico(3)-1).ne.b_all.and.
     &            p_tmp(ico(1),ico(2),ico(3)-1,1).eq.-1)then
                  call push(fill,(/ico(1),ico(2),ico(3)-1/))
               end if
            end if
            p_tmpi(ico(1),ico(2),ico(3)) = b_all
            goto 142
         end if
         !empty fill
         call clear(fill)
         !at end go through all 0's and look around if there are other 0's (for corners)
         
         ! go through all 0's and see if they are adjacent to a filled fluid particle, then fill
         do i=1,mult(1)
         do j=1,mult(2)
         do k=1,mult(3)
            if(p_tmp(i,j,k,1).eq.0)then ! surface particle
               do l=-1,1
               do m=-1,1
               do n=-1,1
                  if(i+l.gt.0.and.i+l.le.mult(1) .and. ! check array boundaries
     &               j+m.gt.0.and.j+m.le.mult(2) .and.
     &               k+n.gt.0.and.k+n.le.mult(3)) then
                     if(p_tmp(i+l,j+m,k+n,1).lt.0.and.
     &                  p_tmpi(i+l,j+m,k+n).ne.0)then ! look for filled fluid particle
                        p_tmpi(i,j,k) = filled ! fill surface particle
                        goto 155
                     end if
                  end if
               end do
               end do
               end do
            end if
155         continue
         end do
         end do
         end do

         min_fluid = 1e8
         max_fluid = -1e8
         do i=1,mult(1)
            do j=1,mult(2)
               do k=1,mult(3)
                  if(p_tmp(i,j,k,1).le.0.and.
     &               p_tmpi(i,j,k).ne.0)then
                     p_tmp(i,j,k,1) = 0
                     if(k*dz.gt.max_fluid) max_fluid = k*dz
                     if(k*dz.lt.min_fluid) min_fluid = k*dz
                  elseif(p_tmp(i,j,k,1).eq.0)then
                     p_tmp(i,j,k,1) = -1
                  end if
               end do
            end do
         end do
         if(abs(max_fluid-min_fluid+dx-h_SWL).gt.1e-4)then
            write(*,*) 'Propsed Still Water Level:', max_fluid-min_fluid+dx
            write(*,*) 'Current Still Water Level:', h_SWL
            write(*,*) 'Replace current with proposed SWL? ',
     &                 '(0 = No, 1 = Yes)'
            read(*,*) i
            write(*,*) i
            if(i.eq.1) h_SWL = max_fluid-min_fluid+dx
            write(*,*) 'h_SWL = ', h_SWL
         end if
         !end universal filling algorithm
         
         ! Periodic open boundaries
         i_periodicOBs(1) = 0
         i_periodicOBs(2) = 0
         i_periodicOBs(3) = 0
154      ico = (/-1,-1,-1/)
         nseed = 0
         do i=1,mult(3)
            do j=1,mult(2)
               do k=1,mult(1)
                  if(p_tmp(k,j,i,1).gt.pob0.and.
     &               p_tmp(k,j,i,1).le.pob1)then
                     ico = (/i,j,mult(3)/) !a-todo: change that later when pob become more general
                     nseed = 1
                     !a-todo: check only in y direction and on borders of domain
                     mym1 = p_tmp(k,modulo((j-2),mult(2))+1,i,1)
                     myp1 = p_tmp(k,modulo((j),mult(2))+1,i,1)
                     if(mym1.gt.pob0.and.mym1.le.pob1)then
                        mym1 = p_tmp(k,modulo((j-3),mult(2))+1,i,1)
                     end if
                     if(myp1.gt.pob0.and.myp1.le.pob1)then
                        myp1 = p_tmp(k,modulo((j+1),mult(2))+1,i,1)
                     end if
                     if((mym1.eq.0.and.myp1.eq.0))then!.or.
!     &                  (mym1.ge.2.and.mym1.le.3..and.
!     &                   myp1.ge.2.and.myp1.le.3))then
                        ico = (/i,j,k/)
                        goto 145
                     end if
                  end if
               end do
            end do
         end do
         if(nseed.eq.0)then
            goto 153
         end if
145      if(ico(1).eq.-1)then
            write(*,*) 'No Seed Point could be found for Periodic ',
     &                     'Boundaries'
            stop
         end if
         i_periodicOBs(1) = 0
         i_periodicOBs(2) = 1
         i_periodicOBs(3) = 0
         ! problems: moving boundaries, corners, non-conforming boundaries
         call pob_init(ico,mult)
         goto 154
         ! Initialise positions
153      np = 0
150      if(mat.ne.-1)then
            mati = getmax(mats)
            rout = get_el(mats,mati)
            mat = rout(1)
            call rep_el(mats,mati,(/-1.0d0/))
            if(mat.gt.2) goto 150
            if(mat.ne.-1) call init_pos(mat,mult,dx,dy,dz,expont,g)
            goto 150
         end if

         write(*,*) ' NBF, NBFM, NB, NP '
         write(*,*) nbf, nbfm, nb, np
         if(.not. empty(mbi))then
            write(*,*) ' Moving Boundary indices (mbi) '
            call wout(mbi)
         end if
         if(.not.empty(fbi))then
            write(*,*) ' Floating Boundary indices (fbi) '
            call wout(fbi)
         end if

         ! Moving boundary treatment
         open(11,file="MULT_MV")
         if(nbf.ne.nbfm)then
            mv_xmin = xp(1)
            mv_xmax = xp(1)
            mv_ymin = yp(1)
            mv_ymax = yp(1)
            mv_zmin = zp(1)
            mv_zmax = zp(1)
            write(11,*) get_len(mbi)
            call wout(mbi)
            do i = 1, get_len(mbi)
               aout = get_el(mbi,i)
               write(11,*) aout(1)
            end do
            if(get_len(mbi).gt.0)then
               write(*,*) ' Select movement pattern for moving 
     &                     boundaries:'
            end if
            do i=1,get_len(mbi)
               write(*,*) ' Moving Boundary Number ',i,'/',get_len(mbi)
               write(*,*) ' First Particle (i,x,y,z) '
              
               aout = get_el(mbi,i)
               write(*,*) aout(1),xp(aout(1)),yp(aout(1)),zp(aout(1))
               write(*,*) ' Last Particle (i,x,y,z) '
               if(i.eq.get_len(mbi))then
                  aout(1) = nbfm-1
               else
                  aout = get_el(mbi,i+1)-1
               end if
               write(*,*) aout(1),xp(aout(1)),yp(aout(1)),zp(aout(1))
               write(*,*) ' Movement pattern: '
               write(*,*) '  (1) linear (e.g. gate) '
               write(*,*) '  (2) sinusoidal (e.g. piston) '
               write(*,*) '  (3) rotation (e.g. flap) '
               write(*,*) '  (4) position file '
               read(*,*) i_mv_type
               write(*,*) i_mv_type
               if(i_mv_type == 1) then
                  write(*,*) ' Specify variables for linear movement:'
                  write(*,*) ' Enter velocity for moving boundaries: '
                  write(*,*) ' (ux, uy, uz) '
                  read(*,*) mv_ux, mv_uy, mv_uz
                  write(*,*) mv_ux, mv_uy, mv_uz
               elseif(i_mv_type == 2) then
                  write(*,*)' Specify variables for sinusoidal movement'
                  write(*,*) ' x(t) = x(0) + a * sin(2*pi/b*t + c) '
                  write(*,*) ' Movement amplitude(a), period(b),
     &                         phase(c)?'
                  read(*,*) mv_amp, mv_per, mv_pha
                  write(*,*) mv_amp, mv_per, mv_pha
                  write(*,*) ' Direction of movement (x,y,z)? '
                  read(*,*) mv_ux,mv_uy,mv_uz
                  write(*,*) mv_ux,mv_uy,mv_uz
               elseif(i_mv_type == 3) then
                  write(*,*)' Specify variables for rotational movement'
                  write(*,*) ' Movement period? '
                  read(*,*) mv_per
                  write(*,*) mv_per
                  write(*,*)' Min and max ellongation angle in degrees?'
                  write(*,*)' If min > max const. rotation is assumed.'
                  read(*,*) mv_min_a, mv_max_a
                  if(mv_min_a.lt.mv_max_a)then
                     if(mv_min_a.gt.0) mv_min_a = -mv_min_a
                     if(mv_max_a.lt.0) mv_max_a = -mv_max_a
                  end if
                  write(*,*) mv_min_a, mv_max_a
                  write(*,*) ' Center of rotation (cx,cy,cz)? '
                  read(*,*) mv_cx, mv_cy, mv_cz
                  write(*,*) mv_cx, mv_cy, mv_cz
                  write(*,*) ' Rotation axis (ax,ay,az)? '
                  read(*,*) mv_ux, mv_uy, mv_uz
                  write(*,*) mv_ux, mv_uy, mv_uz
               elseif(i_mv_type.eq.4) then
                  write(*,*)' Enter index i of IMV_B file to use '
                  write(*,*)' Filename: IMV_B.i '
                  read(*,*) mv_ux
                  write(mv_file,'(i3)') mv_ux
                  mv_file = trim(mv_file)
                  write(mv_file,'(a,i4.4)') 'IMV_B.',mv_file
                  write(*,*)' Reading file '//mv_file//
     &                      ' with data (t, x, y, z)'
                  open(135,file=mv_file)
                  do
                     read(135,*,end=136) t,x,y,z
                     call push(f_mv,(/t,x,y,z/))
                  end do
136               close(135)
               else
                  write(*,*) ' Incorrect Movement type. Stop. '
                  stop
               end if 
               if(i_mv_type.ne.4)then
                  write(*,*) ' Start and stop time for movement? '
                  read(*,*) mv_t0, mv_t1
                  write(*,*) mv_t0, mv_t1
               end if
               !write to file   
               write(11,*)i_mv_type
               write(11,*)mv_ux
               write(11,*)mv_uy
               write(11,*)mv_uz
               write(11,*)mv_amp
               write(11,*)mv_per
               write(11,*)mv_pha
               write(11,*)mv_cx
               write(11,*)mv_cy
               write(11,*)mv_cz
               write(11,*)mv_min_a
               write(11,*)mv_max_a
               write(11,*)mv_t0
               write(11,*)mv_t1
               if(i_mv_type.ne.1) call mv_calc_limits(j,k)
             end do
             close(11)
         else
             close(11,status="delete")
         end if
         ! Calculate computational domain
         mv_xmin = minval(xp(1:nbf))
         mv_xmax = maxval(xp(1:nbf))
         mv_ymin = minval(yp(1:nbf))
         mv_ymax = maxval(yp(1:nbf))
         mv_zmin = minval(zp(1:nbf))
         mv_zmax = maxval(zp(1:nbf))
         write(*,*) ' Limits calculated '
         write(*,*) 'x:',mv_xmin,mv_xmax
         write(*,*) 'y:',mv_ymin,mv_ymax
         write(*,*) 'z:',mv_zmin,mv_zmax
         
         call floating_objects()
      
         ! Calculate pressure at boundaries
         write(*,*) ' Correct pressure at boundaries? '
         read(*,*) i_correct_pb
         write(*,*) i_correct_pb

         if(i_correct_pb.eq.1) then
            XXmin = mv_xmin ! 0.
            XXmax = mv_xmax ! mult(1)*dx
            YYmin = mv_ymin ! 0.
            YYmax = mv_ymax ! mult(2)*dy
            ZZmin = mv_zmin ! 0.
            ZZmax = mv_zmax ! mult(3)*dz
            call correct_p_boundaries(expont,g,XXmin,XXmax,YYmin,YYmax,
     &                                ZZmin,ZZmax,dx,dy,dz)
         else
            call p_boundaries(dx,dy,dz)
         end if
         ! Fill up various files, which are now not needed
         open(55,file='obstacle')
         write(55,*) '0'
         close(55)
         open(66,file='wavemaker')
         write(66,*) '0'
         close(66)
         open(77,file='gate')
         write(77,*) '0'
         close(77)
         open(88,file='Tsunami_Landslide.txt')
         write(88,*) '0'
         close(88)         
         if(allocated(p_tmp)) deallocate(p_tmp)
         if(allocated(p_tmpi)) deallocate(p_tmpi)
         call destroy(fill)
         call destroy(mbi)
         call destroy(fbi)
         call destroy(f_mv)
      end subroutine
      
      subroutine mv_calc_limits(istart,iend)
      
         use mv_vars
         use mv_shared
         include 'common.gen3D'
         
         double precision t0,t1,x,y,z,a,w,c
         double precision :: aout(4)
         double precision theta, len_a, s, x_o,y_o,z_o,t,dt3
         integer it,i
         integer, parameter :: divide = 100
         t0 = mv_t0
         t1 = mv_t1
         if(mv_per+t0.lt.t1) t1 = mv_per+t0
         
         if(i_mv_type.eq.2)then
            write(*,*) 'istart/end', istart, iend
            dt3 = (t1-t0)/real(divide)
            do it = 0,divide
               t = t0 * real(it)*dt3
               a = mv_amp
               w = 2*pi/mv_per
               c = mv_pha
               u_x = mv_ux/sqrt(mv_ux**2+mv_uy**2+mv_uz**2) !unit vector in moving direction
               u_y = mv_uy/sqrt(mv_ux**2+mv_uy**2+mv_uz**2)
               u_z = mv_uz/sqrt(mv_ux**2+mv_uy**2+mv_uz**2)
               do i=istart,iend
                  x=xp(i)+u_x*a*(sin(w*t+c))
                  y=yp(i)+u_y*a*(sin(w*t+c))
                  z=zp(i)+u_z*a*(sin(w*t+c))
                  if(x.lt.mv_xmin) mv_xmin = x
                  if(x.gt.mv_xmax) mv_xmax = x
                  if(y.lt.mv_ymin) mv_ymin = y
                  if(y.gt.mv_ymax) mv_ymax = y
                  if(z.lt.mv_zmin) mv_zmin = z
                  if(z.gt.mv_zmax) mv_zmax = z
               end do
            end do
         elseif(i_mv_type.eq.3)then
            dt3 = 1/real(divide)
            do it = 0,divide
               t = real(it)*dt3
               if(mv_max_a.gt.mv_min_a)then
                  len_a = mv_max_a - mv_min_a
                  theta = (mv_min_a + t*len_a)*pi/180
               else ! mv_max_a < mv_min_a means constant rotation
                  theta = 2*pi*t
               end if
               u_x = mv_ux/sqrt(mv_ux**2+mv_uy**2+mv_uz**2) !unit vector for rotation axis
               u_y = mv_uy/sqrt(mv_ux**2+mv_uy**2+mv_uz**2)
               u_z = mv_uz/sqrt(mv_ux**2+mv_uy**2+mv_uz**2)
               s = sin(theta)
               c = cos(theta)
               do i=istart,iend
                  x_o = xp(i) - mv_cx
                  y_o = yp(i) - mv_cy
                  z_o = zp(i) - mv_cz
                  x = (u_x**2 + (1-u_x**2)*c) * x_o ! according to http://en.wikipedia.org/wiki/Rotation_matrico(1)#Axis_of_a_rotation
                  x = (u_x*u_y*(1-c) - u_z*s) * y_o + x
                  x = (u_x*u_z*(1-c) + u_y*s) * z_o + x + mv_cx
                  y = (u_x*u_y*(1-c) + u_z*s) * x_o
                  y = (u_y**2 + (1-u_y**2)*c) * y_o + y
                  y = (u_y*u_z*(1-c) - u_x*s) * z_o + y + mv_cy
                  z = (u_x*u_z*(1-c) - u_y*s) * x_o
                  z = (u_y*u_z*(1-c) + u_x*s) * y_o + z
                  z = (u_z**2 + (1-u_z**2)*c) * z_o + z + mv_cz
                  if(x.lt.mv_xmin) mv_xmin = x
                  if(x.gt.mv_xmax) mv_xmax = x
                  if(y.lt.mv_ymin) mv_ymin = y
                  if(y.gt.mv_ymax) mv_ymax = y
                  if(z.lt.mv_zmin) mv_zmin = z
                  if(z.gt.mv_zmax) mv_zmax = z
               end do
            end do
         elseif(i_mv_type.eq.4)then
            do i = istart,iend
               x_o = xp(i)
               y_o = yp(i)
               z_o = zp(i)
               do j = 1,get_len(f_mv)
                  aout = get_el(f_mv, j)

                  x = x_o + aout(2)
                  y = y_o + aout(3)
                  z = z_o + aout(4)
                  if(x.lt.mv_xmin) mv_xmin = x
                  if(x.gt.mv_xmax) mv_xmax = x
                  if(y.lt.mv_ymin) mv_ymin = y
                  if(y.gt.mv_ymax) mv_ymax = y
                  if(z.lt.mv_zmin) mv_zmin = z
                  if(z.gt.mv_zmax) mv_zmax = z
               end do
            end do
         end if
      end subroutine
      
      
      subroutine floating_objects()
         use mv_vars
         include 'common.gen3D'

         double precision x1,y1,z1,x2,y2,z2
         double precision xtemp, ytemp, ztemp
         integer i0(1),i1(1)
         real mat, vol, cx, cy, cz, ixx, iyy, izz

         if(num_FB.gt.get_len(fbi))then
            print*,'Number of floating Bodies exceeds max value'
            print*,'num_FB.gt.num_FB_max'
            print*,'Adjust num_FB_max in common.gen3D, num_FB_max = ',
     &               num_FB_max
            stop
         end if
         
         open(88,file='Floating_Bodies.txt')
         if(empty(fbi))then
            write(88,*) 0
            close(88)
            return
         else
            write(88,*) 1
         end if
         write(88,*) nbfm
         open(144,file='FO')
         do num_FB = 1,get_len(fbi)
            read(144,*) mat,vol,cx,cy,cz,ixx,iyy,izz

            XcylinderDimension(num_FB) = 0 !a-todo: these variables are not used in SPHysics, but are written out
            YcylinderDimension(num_FB) = 0
            ZcylinderDimension(num_FB) = 0

            write(*,*) 'Floating object nr. ',num_FB
            write(*,*) 'Material: ',mat
            write(*,*) 'Center of gravity: ',cx,cy,cz

            write(*,*) 'Enter specific Weight'
            read(*,*)  FB_specificWeight(num_FB)   
            write(*,*) FB_specificWeight(num_FB)
            write(*,*) 'Enter (x,y,z) shift of Centre of Gravity '
            write(*,*) 'for use in Parallel Axis Theorem '
            read(*,*)  xdiff_CofG, ydiff_CofG, zdiff_CofG  !Shift Box Centre of Gravity to make slightly unbalanced
            write(*,*) xdiff_CofG, ydiff_CofG, zdiff_CofG

            cylinderDensity(num_FB) = rho0 * FB_specificWeight(num_FB)
            bigMass(num_FB) = vol*cylinderDensity(num_FB)
            bigInertiaXX(num_FB) = ixx*cylinderDensity(num_FB)
            bigInertiaYY(num_FB) = iyy*cylinderDensity(num_FB)
            bigInertiaZZ(num_FB) = izz*cylinderDensity(num_FB)
            Box_XC(num_FB) = cx + xdiff_CofG
            Box_YC(num_FB) = cy + ydiff_CofG
            Box_ZC(num_FB) = cz + zdiff_CofG
            bigInertiaXX(num_FB) = bigInertiaXX(num_FB)
     &            + bigMass(num_FB)
     &            * (ydiff_CofG**2 + zdiff_CofG**2)  !Parallel Axis Theroem
            bigInertiaYY(num_FB) = bigInertiaYY(num_FB)
     &            + bigMass(num_FB)
     &            * (xdiff_CofG**2 + zdiff_CofG**2)  !Parallel Axis Theroem
            bigInertiaZZ(num_FB) = bigInertiaZZ(num_FB)
     &            + bigMass(num_FB)
     &            * (xdiff_CofG**2 + ydiff_CofG**2)  !Parallel Axis Theroem
            
            write(*,*) 'Mass: ',bigMass(num_FB)
            write(*,*) 'Inertia: ',bigInertiaXX(num_FB),
     &                  bigInertiaYY(num_FB),bigInertiaZZ(num_FB)
           	
           	write(*,*) 'Enter initial U,V,W velocity of Object'
            read(*,*)  bigU(num_FB), bigV(num_FB), bigW(num_FB)
            write(*,*) bigU(num_FB), bigV(num_FB), bigW(num_FB)
            write(*,*) 'Enter initial Rotation Rate (Omega)'
            write(*,*) 'in X,Y & Z Directions'
            read(*,*)  bigOmegaX(num_FB), bigOmegaY(num_FB), 
     &                 bigOmegaZ(num_FB)
            write(*,*) bigOmegaX(num_FB), bigOmegaY(num_FB), 
     &                 bigOmegaZ(num_FB)
            write(*,*) 'Enter coefficient of Friction'
            read(*,*)  friction_coeff(num_FB)
            write(*,*) friction_coeff(num_FB)
             
            i0 = get_el(fbi,num_FB)
            if(num_FB.eq.get_len(fbi))then
               i1 = nb
            else
               i1 = get_el(fbi,num_FB+1)
            end if
            write(*,*) i0(1), i1(1)
               
            do i=i0(1),i1(1)
               x = xp(i) - Box_XC(num_FB)
               y = yp(i) - Box_YC(num_FB)
               z = zp(i) - Box_ZC(num_FB)
               u1 = bigU(num_FB) + ( bigOmegaY(num_FB)*z - 
     &                  bigOmegaZ(num_FB)*y )
               v1 = bigV(num_FB) + ( bigOmegaZ(num_FB)*x - 
     &                  bigOmegaX(num_FB)*z )
               w1 = bigW(num_FB) + ( bigOmegaX(num_FB)*y - 
     &                  bigOmegaY(num_FB)*x )
               call pos_veloc(i,xp(i),yp(i),zp(i),u1,v1,w1) 
            end do
             
            nb_FB(num_FB) = i1(1)
            write(*,*) 'Object number: ',num_FB
            write(*,*) 'no. of Object particles: nb_FB',nb_FB(num_FB)

            write(88,*)num_FB
            write(88,*)bigMass(num_FB)
            write(88,*)bigInertiaXX(num_FB),
     &                  bigInertiaYY(num_FB),bigInertiaZZ(num_FB)
            write(88,*)XcylinderDimension(num_FB),
     &                  YcylinderDimension(num_FB),
     &                  ZcylinderDimension(num_FB)
            write(88,*)cylinderDensity(num_FB)
            write(88,*)FB_SpecificWeight(num_FB)
            write(88,*)friction_coeff(num_FB)
            write(88,*)Box_XC(num_FB),Box_YC(num_FB),Box_ZC(num_FB)
            write(88,*)bigU(num_FB),bigV(num_FB),bigW(num_FB),
     &                  bigOmegaX(num_FB),bigOmegaY(num_FB),
     &                  bigOmegaZ(num_FB)
            write(88,*)nb_FB(num_FB)
  	      end do
  	      close(88)
      end subroutine
      
      subroutine pob_init(ico,mult)
         use mv_vars
         integer, intent(in) :: ico(3), mult(3)
         logical in_old, in_start, in
         real prev(3,2)
         in = .true.
         prev(1,1) = p_tmp(ico(1),modulo(ico(2)-2,mult(2))+1,ico(3),1)
         prev(1,2) = p_tmp(ico(1),modulo(ico(2),mult(2))+1,ico(3),1)
         if(prev(1,1).gt.pob0.and.prev(1,1).le.pob1)then
            prev(1,1) =p_tmp(ico(1),modulo(ico(2)-3,mult(2))+1,ico(3),1)
         end if
         if(prev(1,2).gt.pob0.and.prev(1,2).le.pob1)then
            prev(1,2) =p_tmp(ico(1),modulo(ico(2)+1,mult(2))+1,ico(3),1)
         end if
         prev(3,:) = prev(1,:)
         do i = ico(1),1,-1
            prev(2,:) = prev(1,:)
            do j = ico(3),1,-1
               call pob_init_ijk(i,ico(2),j,mult, in, prev(1,:))
               if(j.eq.ico(3)) in_start = in
            end do
            prev(1,:) = prev(2,:)
            in = in_start
            do j = ico(3)+1,mult(3)
               call pob_init_ijk(i,ico(2),j,mult, in, prev(1,:))
            end do
            in = in_start
            prev(1,:) = prev(2,:)
         end do
         in = .true.
         prev(1,:) = prev(3,:)
         do i = ico(1)+1,mult(1)
            prev(2,:) = prev(1,:)
            do j = ico(3),1,-1
               call pob_init_ijk(i,ico(2),j,mult, in, prev(1,:))
               if(j.eq.ico(3)) in_start = in
            end do
            prev(1,:) = prev(2,:)
            in = in_start
            do j = ico(3)+1,mult(3)
               call pob_init_ijk(i,ico(2),j,mult, in, prev(1,:))
            end do
            in = in_start
            prev(1,:) = prev(2,:)
         end do
      end subroutine
      
      subroutine pob_init_ijk(i,j,k,mult, in, prev)
         use mv_vars
         integer, intent(in) :: i,j,k,mult(3)
         real mym1, myp1
         logical, intent(inout) :: in
         real, intent(inout) :: prev(1,2)
         mym1 = p_tmp(i,modulo((j-2),mult(2))+1,k,1)
         myp1 = p_tmp(i,modulo((j),mult(2))+1,k,1)
         if(mym1.gt.pob0.and.mym1.le.pob1)then
            mym1 = p_tmp(i,modulo((j-3),mult(2))+1,k,1)
         end if
         if(myp1.gt.pob0.and.myp1.le.pob1)then
            myp1 = p_tmp(i,modulo((j+1),mult(2))+1,k,1)
         end if
         if(mym1.le.0.and.prev(1,1).ge.1.and.prev(1,1).le.3)then
            in = .not.in
         end if
         if(myp1.le.0.and.prev(1,2).ge.1.and.prev(1,2).le.3)then
            in = .not.in
         end if
         if(in)then
            if(mym1.eq.-1.or.myp1.eq.-1)then
               p_tmp(i,j,k,1) = -1
            elseif(mym1.eq.0.or.myp1.eq.0)then
               p_tmp(i,j,k,1) = 0
            elseif(mym1.eq.myp1)then
               p_tmp(i,j,k,1) = mym1
            else
               p_tmp(i,j,k,1) = 2
            end if
         else
            p_tmp(i,j,k,1) = 2
         end if
         prev(1,:) = (/mym1,myp1/)
      end subroutine
      
      subroutine init_pos(mat,mult,dx,dy,dz,expont,g)
         use mv_vars
         include 'common.gen3D'
         real, intent(in) :: mat
         integer, intent(in) :: mult(3)
         nn = np
         if(mat.eq.2)then !fixed boundaries
         do k = 1,mult(3)
            do i = 1,mult(1)
               do j = 1, mult(2)
                  if(p_tmp(i,j,k,1)>=2.and.p_tmp(i,j,k,1)<=3)then
                     nn = nn + 1
                     p_tmp(i,j,k,1) = - p_tmp(i,j,k,1) + 4.
                     if(f_zmax.lt.(k-1)*dz) f_zmax = (k-1)*dz
                     call pos_veloc(nn, (i-1)*dx,(j-1)*dy,(k-1)*dz,
     &                              0.,0.,0.)
                     if(iBC.eq.2)then
                        nn = nn + 1
                        call pos_veloc(nn, (i-1.5)*dx,(j-1.5)*dy,
     &                                 (k-1.5)*dz,0.,0.,0.)
                     end if
                  end if
               end do
            end do
         end do
         else !other particles
         do k = 1,mult(3)
            do i = 1,mult(1)
               do j = 1, mult(2)
                  if(p_tmp(i,j,k,1).eq.mat)then
                     nn = nn + 1
                     call pos_veloc(nn, (i-1)*dx,(j-1)*dy,(k-1)*dz,
     &                              0.,0.,0.)
                     if(f_zmax.lt.(k-1)*dz.and.mat.ne.0)f_zmax=(k-1)*dz
                     if(mat.eq.0) call pressure(nn,f_zmax,dx,dy,dz,
     &                                          expont,g)
                     if((mat.eq.0.and.lattice.eq.2).or.
     &                  (mat.ge.1.and.iBC.eq.2))then
                        nn = nn + 1
                        call pos_veloc(nn, (i-1.5)*dx,(j-1.5)*dy,
     &                                 (k-1.5)*dz,0.,0.,0.)
                        if(mat.eq.0) call pressure(nn,f_zmax,dx,dy,
     &                                             dz,expont,g)
                     end if
                  end if
               end do
            end do
         end do
         end if
         if(nn.gt.npar)then
            write(*,*) 'Number of Particles (',nn,') exceeds npar. ',
     &                 'Change value in common.gen3D. Stop.'
            stop
         end if
         if(mat.eq.2)then
            nbf = nn
         elseif(mat.ge.1)then
            call push(mbi,(/np/))
            nbfm = nn
         elseif(mat.gt.0)then
            call push(fbi,(/np/))
            nb = nn
         elseif(mat.eq.0)then
            if(nbfm.eq.-1) nbfm = nbf
            if(nb.eq.-1) nb = nbfm
         else
            write(*,*) 'Unkown mat: ',mat,'. Stop'
            stop
         end if
         np = nn
      end subroutine
!arno end
