#!BPY

"""
Name: 'SPHysics Export v3.4'
Blender: 249
Group: 'Export'
Tooltip: 'Transforms a mesh into a point cloud and exports it for use with SPHysics'
"""

__author__ = 'Arno Mayrhofer'
__url__ = 'http://www.amconception.de'
__version__ = '3.4'

# License: GPL
# Written by Arno Mayrhofer, 2009-2010, University of Vienna, Austria
# http://sci.amconception.de

import Blender
import bpy
import math
import random
from Blender import Mathutils
from Blender.Mathutils import *
import time
import sys
import os
import string

acc = '.10f' # output of variables to 5 points after decimal point
dh = 0
eps = 1E-17


# Variables for volume calculation according to http://www.cs.berkeley.edu/~mirtich

A = 0 #/* alpha */
B = 0 #/* beta */
C = 0 #/* gamma */

#/* projection integrals */
P1, Pa, Pb, Paa, Pab, Pbb, Paaa, Paab, Pabb, Pbbb = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

#/* face integrals */
Fa, Fb, Fc, Faa, Fbb, Fcc, Faaa, Fbbb, Fccc, Faab, Fbbc, Fcca = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

#/* volume integrals */
T0, T1, T2, TP = 0, [0, 0, 0], [0, 0, 0], [0, 0, 0]

def check_pob(mesh): # check if every face of a periodic boundary has a twin
	fs = mesh.faces
	cor = [] #corresponding face
	trans = []
	for f in fs:
		cor.append(-1)
	for m in mesh.materials:
		trans.append([0,0,0])
	for i_f in xrange(len(fs)):
		if(cor[i_f]!=-1):
			continue # already found twin
		n_vs = len(fs[i_f].verts)
		vs = fs[i_f].verts
		for f in fs:
			vs2 = f.verts
			if(f.index == i_f or len(f.verts)!=n_vs):
				continue
			for i in xrange(n_vs):
				off = vs[0].co - vs2[i].co
				#a-todo: limitation for now only y-translation is allowed / future: translation along normal
				if(off[0] != 0 and off[2] != 0):
					continue
				for j in xrange(n_vs):
					found = 0
					for k in xrange(n_vs):
						if(vs[j].co - vs2[k].co == off):
							found = 1
					if(found == 0):
						break
					#if(vs[j].co - vs2[(i+j)%n_vs].co != off):
					#	found = 0
					#	break
				if(found == 1):
					if((trans[f.mat] != off or trans[f.mat] != -1*off) and trans[f.mat] != [0, 0, 0]):
						print 'Not all faces of color ', mesh.materials[f.mat].rgbCol, 'share the same offset'
						print 'STOP'
						return
					if(trans[f.mat] == [0,0,0]):
						trans[f.mat] = off
					cor[i_f] = f.index
					cor[f.index] = i_f
					break
		if(cor[i_f]==-1):
			print 'Could not find a suitable twin for face nr. ', i_f
			print 'STOP'
			return
		#a-todo: this has to be extended as soon as SPHysics offers certain capabilities.
	return

def merge(nc):
	global path
	t0 = time.time()
	fname = path+'IPART_B'+str(1)
	data = open(fname)
	mult = [0, 0, 0]
	dx = float(data.readline())
	dy = float(data.readline())
	dz = float(data.readline())
	mult[0] = int(data.readline())
	mult[1] = int(data.readline())
	mult[2] = int(data.readline())
	n = [0,0,0]
	part = {}
	for ic in xrange(1,nc+1):
		if(ic!=1):
			fname = path+'IPART_B'+str(ic)
			data = open(fname)
		for d in data:
			d = string.split(d)
			[x,y,z,mat,n[0],n[1],n[2]] = [int(float(d[0])), int(float(d[1])), int(float(d[2])), float(d[3]), float(d[4]), float(d[5]), float(d[6])]
			if((x,y,z) in part):
				mat_o = part[(x,y,z)][0]
				if (mat_o == 2 and mat <2 and mat >= 1) or (mat == 2 and mat_o <2 and mat_o >= 1): # this is for combined fixed and moving particles
					part[(x,y,z)][0] = -(mat_o+mat) + 6. # normals are already set, maybe use average later on
				elif(mat_o < mat):
					part[(x,y,z)] = [mat,n[0],n[1],n[2]]
			else:
				part[(x,y,z)] = [mat,n[0],n[1],n[2]]
	np = len(part)
	#output
	# part header structure:
	# dx
	# dy
	# dz
	# np
	# mult[0]
	# mult[1]
	# mult[2]
	part_out = file(path+'IPART_B','w')
	part_out.write(str(dx) + '\n' + str(dy) + '\n' + str(dz) + '\n' + str(np) + '\n' + str(mult[0]) + '\n' + str(mult[1]) + '\n' + str(mult[2]) + '\n\n')
	acc = '.5f' # output of variables to 5 points after decimal point
	for (k,v) in sorted(part.iteritems(), key=lambda(k,v): (-v[0],k[2],k[0],k[1])): #parts:
		out = ('%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc) % (k[0]*dx,k[1]*dy,k[2]*dz,v[0],v[1],v[2],v[3])
		part_out.write(out + '\n')
	t0 = time.time()-t0
	part_out.close()
	print('Merging finished in '+str(t0)+' s')
	return

def compProjectionIntegrals(t):
	global P1, Pa, Pb, Paa, Pab, Pbb, Paaa, Paab, Pabb, Pbbb
	P1, Pa, Pb, Paa, Pab, Pbb, Paaa, Paab, Pabb, Pbbb = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	for i in xrange(3):
		a0 = t[i][A]
		b0 = t[i][B]
		a1 = t[(i+1)%3][A]
		b1 = t[(i+1)%3][B]

		da = a1 - a0
		db = b1 - b0
		a0_2 = a0 * a0
		a0_3 = a0_2 * a0
		a0_4 = a0_3 * a0
		b0_2 = b0 * b0
		b0_3 = b0_2 * b0
		b0_4 = b0_3 * b0
		a1_2 = a1 * a1
		a1_3 = a1_2 * a1
		b1_2 = b1 * b1
		b1_3 = b1_2 * b1

		C1 = a1 + a0
		Ca = a1*C1 + a0_2
		Caa = a1*Ca + a0_3
		Caaa = a1*Caa + a0_4
		Cb = b1*(b1 + b0) + b0_2
		Cbb = b1*Cb + b0_3
		Cbbb = b1*Cbb + b0_4;
		Cab = 3*a1_2 + 2*a1*a0 + a0_2
		Kab = a1_2 + 2*a1*a0 + 3*a0_2
		Caab = a0*Cab + 4*a1_3
		Kaab = a1*Kab + 4*a0_3
		Cabb = 4*b1_3 + 3*b1_2*b0 + 2*b1*b0_2 + b0_3
		Kabb = b1_3 + 2*b1_2*b0 + 3*b1*b0_2 + 4*b0_3

		P1 += db*C1
		Pa += db*Ca
		Paa += db*Caa
		Paaa += db*Caaa
		Pb += da*Cb
		Pbb += da*Cbb
		Pbbb += da*Cbbb
		Pab += db*(b1*Cab + b0*Kab)
		Paab += db*(b1*Caab + b0*Kaab)
		Pabb += da*(a1*Cabb + a0*Kabb)

	P1 /= 2.0;
	Pa /= 6.0;
	Paa /= 12.0;
	Paaa /= 20.0;
	Pb /= -6.0;
	Pbb /= -12.0;
	Pbbb /= -20.0;
	Pab /= 24.0;
	Paab /= 60.0;
	Pabb /= -60.0;
	
def compFaceIntegrals(t):
	global Fa,Fb,Fc,Faa,Fbb,Fcc,Faaa,Fbbb,Fccc,Faab,Fbbc,Fcca
	w = -t[3][0]*t[0][0] - t[3][1]*t[0][1] - t[3][2]*t[0][2]
	n = t[3]
	
	compProjectionIntegrals(t)
	
	k1 = 1./n[C]
	k2 = k1**2
	k3 = k1**3
	k4 = k1**4
	
	Fa = k1*Pa
	Fb = k1*Pb
	Fc = -k2 * (n[A]*Pa + n[B]*Pb + w*P1)

	Faa = k1 * Paa
	Fbb = k1 * Pbb
	Fcc = k3 * (n[A]**2*Paa + 2*n[A]*n[B]*Pab + n[B]**2*Pbb + w*(2*(n[A]*Pa + n[B]*Pb) + w*P1))

	Faaa = k1 * Paaa
	Fbbb = k1 * Pbbb
	Fccc = -k4 * (n[A]**3*Paaa + 3*n[A]**2*n[B]*Paab + 3*n[A]*n[B]**2*Pabb + n[B]**3*Pbbb + 3*w*(n[A]**2*Paa + 2*n[A]*n[B]*Pab + n[B]**2*Pbb) + w*w*(3*(n[A]*Pa + n[B]*Pb) + w*P1))

	Faab = k1 * Paab
	Fbbc = -k2 * (n[A]*Pabb + n[B]*Pbbb + w*Pbb)
	Fcca = k3 * (n[A]**2*Paaa + 2*n[A]*n[B]*Paab + n[B]**2*Pabb + w*(2*(n[A]*Paa + n[B]*Pab) + w*Pa));

def calculate_fo(fo_in,mesh):
	global A,B,C,T0,T1,T2,TP
	print('In calculate_fo')
	faces = mesh.faces
	verts = mesh.verts
	T0, T1, T2, TP = 0, [0, 0, 0], [0, 0, 0], [0, 0, 0]
	fo_out = [0, 0, 0, 0, 0, 0, 0]
	#triangulate
	tris = []
	null = Blender.Mathutils.Vector(0,0,0)
	for f in faces:
		f.sel = 0
	for i in fo_in:
		f = faces[i]
		f.sel = 1
	mesh.recalcNormals(0)
	for i in fo_in:
		f = faces[i]
		if(len(f.v)==3):
			tris.append([f.v[0].co, f.v[1].co, f.v[2].co, f.no])
		else:
			i_0 = 1e10
			i_a = -1
			i_b = -1
			i_c = -1
			for v in f.verts:
				if(i_0 > v.index):
					i_0 = v.index
			for e in f.edge_keys:
				if(min(e)==i_0):
					if(i_a<0):
						i_a = max(e)
					else:
						i_b = max(e)
			for v in f.verts:
				j = v.index
				if(j!=i_a and j!=i_b and j!=i_0):
					i_c = j
			tris.append([verts[i_0].co, verts[i_a].co, verts[i_b].co, f.no])
			tris.append([verts[i_c].co, verts[i_a].co, verts[i_b].co, f.no])
	for i in xrange(len(tris)):
		v1 = null
		v2 = null
		v1 = tris[i][1] - tris[i][0]
		v2 = tris[i][2] - tris[i][1]
		norm = v1.cross(v2)
		norm = norm.normalize()
		if((norm-tris[i][3]).length>0.01):
			tmp = tris[i][0]
			tris[i][0]=tris[i][2]
			tris[i][2]=tmp
	print('Start calculating FO')
	for t in tris:
		nx = abs(t[3][0])
		ny = abs(t[3][1])
		nz = abs(t[3][2])
		
		if (nx > ny and nx > nz):
			C = 0
		elif(ny > nz):
			C = 1
		else:
			C = 2

		A = (C + 1) % 3;
		B = (A + 1) % 3;
		
		compFaceIntegrals(t)
		if(A==0):
			T0 += t[3][0]*Fa
		elif(B==0):
			T0 += t[3][0]*Fb
		else:
			T0 += t[3][0]*Fc

		T1[A] += t[3][A] * Faa;
		T1[B] += t[3][B] * Fbb;
		T1[C] += t[3][C] * Fcc;
		T2[A] += t[3][A] * Faaa;
		T2[B] += t[3][B] * Fbbb;
		T2[C] += t[3][C] * Fccc;
		TP[A] += t[3][A] * Faab;
		TP[B] += t[3][B] * Fbbc;
		TP[C] += t[3][C] * Fcca;

	T1[0] /= 2
	T1[1] /= 2
	T1[2] /= 2
	T2[0] /= 3
	T2[1] /= 3
	T2[2] /= 3
	TP[0] /= 2
	TP[1] /= 2
	TP[2] /= 2
	fo_out[0] = T0
	fo_out[1] = T1[0] / T0
	fo_out[2] = T1[1] / T0
	fo_out[3] = T1[2] / T0
	fo_out[4] = T2[1]+T2[2] - T0*(fo_out[2]**2 + fo_out[3]**2)
	fo_out[5] = T2[2]+T2[0] - T0*(fo_out[3]**2 + fo_out[1]**2)
	fo_out[6] = T2[0]+T2[1] - T0*(fo_out[1]**2 + fo_out[2]**2)
	#a-note: if the normals point outward the mass will be negative thus correction is needed
	if(T0 < 0.):
		print 'change'
		print T0
		fo_out[0] *= -1
		fo_out[4] *= -1
		fo_out[5] *= -1
		fo_out[6] *= -1
	return fo_out
	
def special_objects(mesh,fo_file): #floating objects + open boundaries #a-todo this could be done in a parallel fashion as well
	fo_mat = []
	faces = mesh.faces
	mm = mesh.materials
	pob_mesh = bpy.data.meshes.new() # periodic open boundary mesh
	pob_obj = bpy.data.scenes.active.objects.new(pob_mesh)
	pob_verts = pob_mesh.verts
	pob_faces = pob_mesh.faces
	pob_mesh.materials = mm
	for f in faces:
		matc = mm[f.mat].rgbCol
		mata = mm[f.mat].alpha
		if(mata==1):
			mat = 1-matc[2]+matc[0]+matc[1] # fluid = (0,0,1), floating = (0,a,1), moving = (b,0,0), fixed = (1,0,0), a in ]0,1[, b in [0,1[
		elif(mata==0.5):
			if(matc[0]>0):
				mat = 2+matc[0] #source = red (hot)
			elif(matc[1]>0):
				mat = 3+matc[1] #periodic = green (intermediate)
			elif(matc[2]>0):
				mat = 4+matc[2] #sink = blue (cold)
		if(mat>0. and mat<1.):
			fo_mat.append([mat, f.index])
		if(mat>3. and mat<4.): #only periodic boundaries
			vs = f.verts
			for v in vs:
				pob_verts.extend(v.co)
			vs_sel = pob_verts.selected()
			new_fs = pob_faces.extend(vs_sel, indexList=True)
			# transfer material
			for i in vs_sel:
				pob_verts[i].sel = 0
			for new_f in new_fs:
				pob_faces[new_f].mat = f.mat
	check_pob(pob_mesh)

	if(len(fo_mat)==0): 
		print 'No Floating Objects'
		return
	fo_mat.sort()
	fo_ind = []
	j = -1
	for i in xrange(len(fo_mat)):
		if(j!=fo_mat[i][0]):
			j = fo_mat[i][0]
			fo_ind.append(i)
	fo_ind.append(len(fo_mat))

	for i in xrange(len(fo_ind)-1):
		fo_in = []
		for j in xrange(fo_ind[i],fo_ind[i+1]):
			fo_in.append(fo_mat[j][1])
		fo_out = calculate_fo(fo_in,mesh) # array with 7 elements: volume, center of gravity (x,y,z), inertia (xx,yy,zz)
		fo_str = ('%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\n') % (fo_mat[fo_ind[i]][0],fo_out[0],fo_out[1],fo_out[2],fo_out[3],fo_out[4],fo_out[5],fo_out[6])
		fo_file.write(fo_str)

	return

def sphysics_export():
	global dh,path
	if(len(sys.argv)!=11):
		print sys.argv
		print('Unexpected amount of arguments')
		return
	ic = int(sys.argv[5][2:]) #number of core
	nc = int(sys.argv[6][2:]) #total number of cores
	dx = float(sys.argv[7][2:])
	dy = float(sys.argv[8][2:])
	dz = float(sys.argv[9][2:])
	path = sys.argv[10][2:]
	filepath=path+'IPART_B'+str(ic)
	print('\nCore '+str(ic)+'/'+str(nc)+' started.')
	t0 = time.time() #start timing
	fabs = math.fabs
	# calculate dx,dy,dz to 3 decimals precisely
	i = 0
	while (dx < 100):
		dx = dx*10
		i = i+1
	dx = math.floor(dx+0.5)
	dx = dx/10.**i
	i = 0
	while (dy < 100):
		dy = dy*10
		i = i+1
	dy = math.floor(dy+0.5)
	dy = dy/10.**i
	i = 0
	while (dz < 100):
		dz = dz*10
		i = i+1
	dz = math.floor(dz+0.5)
	dz = dz/10.**i
	dxt = dx*0.99
	dyt = dy*0.99
	dzt = dz*0.99
	dh = min(dxt, min(dyt,dzt))
	if(i>5):
		print('Warning: Desired accuracy might not be reached. Adjust output if necessary.')

	lst = list(bpy.data.meshes)
	if not lst:
		print('Error 1: Could not find mesh object.')
		return

	#check materials (only red, black, blue)
	fixed = -1 # in the list we will have fluid = 0, moving = 1, fixed = 2
	fluid = -1
	for mesh in lst:
		for i in xrange(len(list(mesh.materials))):
			if(not mesh.materials[i]):
				continue
			if mesh.materials[i].rgbCol == [1, 0, 0]:
				fixed = i
			elif mesh.materials[i].rgbCol == [0, 0, 1]:
				fluid = i
	if fixed == -1 or fluid == -1:
		print('Error 2: Not all needed colors (blue, red) could be found.')
		return
		
	#calculate size
	gmin = Vector([10000,10000,10000])
	gmax = Vector([-10000,-10000,-10000])
	#the max/min code would need to be executed only once, but it runs very fast so no need for extra treatment (max 1.5 s for 150000 verts)
	for mesh in lst:
		vs = mesh.verts
		for v in vs:
			if(v.co[0] < gmin[0]):
				gmin[0] = v.co[0]
			if(v.co[0] > gmax[0]):
				gmax[0] = v.co[0]
			if(v.co[1] < gmin[1]):
				gmin[1] = v.co[1]
			if(v.co[1] > gmax[1]):
				gmax[1] = v.co[1]
			if(v.co[2] < gmin[2]):
				gmin[2] = v.co[2]
			if(v.co[2] > gmax[2]):
				gmax[2] = v.co[2]
	print gmax
	print gmin
	for mesh in lst:
			for v in mesh.verts: # move to 0
				v.co = v.co - gmin
	gmax = gmax - gmin
	mult = [0, 0, 0]
	mult[0] = int(math.floor(gmax[0]/dx+0.01)+1)
	mult[1] = int(math.floor(gmax[1]/dy+0.01)+1)
	mult[2] = int(math.floor(gmax[2]/dz+0.01)+1)
	if(ic==1):
		print 'Array Dimensions (x,y,z): '
		print mult
	# initialise data
	part = {}
	new_mesh = bpy.data.meshes.new()
	new_obj = bpy.data.scenes.active.objects.new(new_mesh)
	nm_verts = new_mesh.verts
	first_mesh = lst[0]
	flst = range(len(lst))
	for i in xrange(len(lst)):
		flst[i] = len(lst[i].faces)
		if(i>0):
			flst[i]+=flst[i-1]
	# start the big loop through faces assigned to core
	nfa = int(math.ceil(flst[-1]/float(nc)))
	ifa = nfa*(ic-1)
	nfa = nfa*ic
	if(nfa>flst[-1]):
		nfa = flst[-1]
	mesh = first_mesh
	mm = mesh.materials
	mf = mesh.faces
	imod = int(math.ceil(float(nfa-ifa+1)/10.))
	meshi = 0
	for i in xrange(ifa,nfa):
		old_meshi = meshi
		while(flst[meshi]<=i):
			meshi += 1
		if(old_meshi!=meshi):
			first_mesh = lst[meshi]
			mesh = first_mesh
			mm = mesh.materials
			mf = mesh.faces
		if((i-ifa+1)%imod==0): #a-debug
			print 'Core ',ic,' status: ', int(float(i-ifa)/float(nfa-ifa)*100.),'%' #a-debug
		cont = 0
		orig_tall = 0
		mesh.sel = 0
		if(meshi>0):
			ii = i-flst[meshi-1]
		else:
			ii = i
		vs = mf[ii].verts
		edges = mf[ii].edge_keys
		matc = mm[mf[ii].mat].rgbCol
		mata = mm[mf[ii].mat].alpha
		if(mata==1):
			mat = 1-matc[2]+matc[0]+matc[1] # fluid = (0,0,1), floating = (0,a,1), moving = (b,0,0), fixed = (1,0,0), a in ]0,1[, b in [0,1[
		elif(mata==0.5):
			if(matc[0]>0):
				mat = 3+matc[0] #source = red (hot)
			elif(matc[1]>0):
				mat = 4+matc[1] #periodic = green (intermediate)
			elif(matc[2]>0):
				mat = 5+matc[2] #sink = blue (cold)

		n = mf[ii].no
		# check if first face is small enough already
		for ed in edges:
			v1 = first_mesh.verts[ed[0]].co
			v2 = first_mesh.verts[ed[1]].co
			if (v1-v2).length > dh:
					orig_tall = 1
					break
		if(orig_tall): # no
			# create a new mesh with only one face
			nm_verts.delete(nm_verts)
			for v in vs:
				nm_verts.extend(v.co)
			new_mesh.faces.extend(nm_verts)
			#mesh.faces.delete(1,[0]) #no longer needed in multicore
			mesh = new_mesh
			subdivide = mesh.subdivide
			mesh.sel = 1
			while(len(mesh.faces)>0):
				# check if first face is small enough
				edges = mesh.faces[0].edge_keys
				cont = 0
				for ed in edges:
					v1 = mesh.verts[ed[0]].co
					v2 = mesh.verts[ed[1]].co
					if (v1-v2).length > dh: # fabs(v1[0]-v2[0])>dxt or fabs(v1[1]-v2[1])>dyt or fabs(v1[2]-v2[2])>dzt:
							cont = 1
							break
				if(cont): # no, do subdivide
					subdivide()
				else: # yes
					f2del = []
					cont = 0
					faces = mesh.faces
					for j in xrange(len(faces)): # search for all faces small enough
						edges = faces[j].edge_keys
						for ed in edges:
							v1 = mesh.verts[ed[0]].co
							v2 = mesh.verts[ed[1]].co
							if (v1-v2).length > dh: # fabs(v1[0]-v2[0])>dxt or fabs(v1[1]-v2[1])>dyt or fabs(v1[2]-v2[2])>dzt:
									cont = 1
									break
						if(not cont):
							f2del.insert(0,j)
					faces.delete(1,f2del) # and delete them (verts are remaining) then start again until no faces left
			vs = mesh.verts

		# add all vertices to regular grid
		for v in vs:
			co = v.co
			x = int(co[0]/dx+0.01) # 0.01 is 1% inaccuracy due to float, could maybe be further reduced
			y = int(co[1]/dy+0.01) # value always positive so int cast is identical to floor
			z = int(co[2]/dz+0.01)
			if((x,y,z) in part):
				mat_o = part[(x,y,z)][0]
				if (mat_o == 2 and mat <2 and mat >= 1) or (mat == 2 and mat_o <2 and mat_o >= 1): # this is for combined fixed and moving particles
					part[(x,y,z)][0] = -(mat_o+mat) + 6.  # normals are already set, maybe use average later on
				elif(mat_o < mat):
					part[(x,y,z)] = [mat,n[0],n[1],n[2]]
			else:
				part[(x,y,z)] = [mat,n[0],n[1],n[2]]
	# end big loop

	np = len(part)

	#output
	# part header structure: (only if ic == 1)
	# dx
	# dy
	# dz
	# np
	# mult[0]
	# mult[1]
	# mult[2]
	part_out = file(filepath,'w')
	if(ic==1):
		part_out.write(str(dx) + '\n' + str(dy) + '\n' + str(dz) + '\n' + str(mult[0]) + '\n' + str(mult[1]) + '\n' + str(mult[2]) + '\n')
	for (k,v) in sorted(part.iteritems(), key=lambda(k,v): (-v[0],k[2],k[0],k[1])): #parts:
		out = ('%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc+'\t%'+acc) % (k[0],k[1],k[2],v[0],v[1],v[2],v[3])
		part_out.write(out + '\n')
	t1 = time.time()-t0
	part_out.close()
	print('Core '+str(ic)+'/'+str(nc)+' execution time: '+str(t1)+' s')
	done = file(path+'core'+str(ic),'w')
	done.write('done')
	done.close()
	if(ic==1):
		finished = 1
		while finished < nc:
			if(os.access(path+'core'+str(finished+1),os.F_OK)):
				finished += 1
		print('Point generation finished\nMerging files...')
		merge(nc)
		for i in range(nc):
			os.remove(path+'core'+str(i+1))
			os.remove(path+'IPART_B'+str(i+1))
		print('Calculating Special Objects')
		fo_file = open(path+'FO','w')
		for mesh in lst:
			special_objects(mesh,fo_file)
		fo_file.close()
		print('Export finished')
		t1 = time.time()-t0
		print('Full execution time: '+str(t1)+' s')
	return

# Main function call

sphysics_export()
