#!BPY

"""
Name: 'SPHysics Export v3.4'
Blender: 249
Group: 'Export'
Tooltip: 'Transforms a mesh into a point cloud and exports it for use with SPHysics'
"""

__author__ = 'Arno Mayrhofer'
__url__ = 'http://www.amconception.de'
__version__ = '3.4'

# License: GPL
# Written by Arno Mayrhofer, 2009-2010, University of Vienna, Austria
# http://sci.amconception.de

import bpy
import sys
import os
import Blender
import subprocess
from subprocess import Popen, PIPE
from Blender import Draw

def main():
	PREF_DX= Draw.Create(0.1)
	PREF_DY= Draw.Create(0.1)
	PREF_DZ= Draw.Create(0.1)
	PREF_EQ= Draw.Create(1)
	
	pup_block = [\
	('dx:', PREF_DX, 0.0001, 1000, 'Spatial discretization in x direction.'),\
	('dy:', PREF_DY, 0.0001, 1000, 'Spatial discretization in y direction.'),\
	('dz:', PREF_DZ, 0.0001, 1000, 'Spatial discretization in z direction.'),\
	('dy = dx, dz = dx', PREF_EQ, 'If set value of dx is used for dy and dz as well.'),\
	]
	
	if not Draw.PupBlock("SPHysics Export", pup_block):
		return
	
	dx = PREF_DX.val
	if(PREF_EQ==1):
		dy = dx
		dz = dx
	else:
		dy = PREF_DY.val
		dz = PREF_DZ.val
	print('dx,dy,dz',dx,dy,dz)
	path = Blender.Get('filename')
	dirpath = Blender.sys.dirname(path)
	osv = sys.platform
	if(osv=='win32'):
		dirpath += '\\'
		nproci = int(os.getenv('NUMBER_OF_PROCESSORS',1))
		for i in range(1,nproci+1):
			if(os.path.exists(dirpath+'core'+str(i))):
				os.remove(dirpath+'core'+str(i))
			if(os.path.exists(dirpath+'IPART_B'+str(i))):
				os.remove(dirpath+'IPART_B'+str(i))
		if(os.path.exists(dirpath+'IPART_B')):
			os.remove(dirpath+'IPART_B')
		if(os.path.exists(dirpath+'FO')):
			os.remove(dirpath+'FO')
		I_file = open(dirpath+'IPART_B','w')
		I_file.close()
		#subprocess.call(['echo','""','>','"'+dirpath+'IPART_B"'])
		for i in range(1,nproci+1):
			try:
				args = ['blender','-b',path,'-P',dirpath+'..\\..\\source\\blender\\sphysics_export.py','--'+str(i),'--'+str(nproci),'--'+str(dx),'--'+str(dy),'--'+str(dz), '--'+dirpath]
				subprocess.Popen(args)
			except StandardError:
				try:
					bexec = Blender.Get('homedir')
					bexec = bexec[:-8]
					bexec += 'blender'
					args = [bexec,'-b',path,'-P',dirpath+'..\\..\\source\\blender\\sphysics_export.py','--'+str(i),'--'+str(nproci),'--'+str(dx),'--'+str(dy),'--'+str(dz), '--'+dirpath]
					subprocess.Popen(args)
				except StandardError:
					print('Could not find Blender executable.')
					print('Please make sure your installation is correct.')
					print('STOP')
					return
	else: # Only tested on Linux (Ubuntu 10.4) (osv = 'linux2')
		dirpath += '/'
		p1 = Popen(['cat','/proc/cpuinfo'],stdout=PIPE)
		p2 = Popen(['grep','^processor'],stdin=p1.stdout,stdout=PIPE)
		p3 = Popen(['wc','-l'],stdin=p2.stdout,stdout=PIPE)
		nproc = p3.communicate()[0]
		nproci = int(nproc)
		print('Nr. of processors used for SPHysics Export: '+str(nproci))
		for i in range(1,nproci+1):
			if(os.path.exists(dirpath+'core'+str(i))):
				os.remove(dirpath+'core'+str(i))
			if(os.path.exists(dirpath+'IPART_B'+str(i))):
				os.remove(dirpath+'IPART_B'+str(i))
		if(os.path.exists(dirpath+'IPART_B')):
			os.remove(dirpath+'IPART_B')
		if(os.path.exists(dirpath+'FO')):
			os.remove(dirpath+'FO')
		subprocess.call(['touch',dirpath+'IPART_B'])
		for i in range(1,nproci+1):
			try:
				args = ['blender-bin','-b',path,'-P',dirpath+'../../source/blender/sphysics_export.py','--'+str(i),'--'+str(nproci),'--'+str(dx),'--'+str(dy),'--'+str(dz), '--'+dirpath]
				subprocess.Popen(args)
			except StandardError:
				try:
					args = ['blender','-b',path,'-P',dirpath+'../../source/blender/sphysics_export.py','--'+str(i),'--'+str(nproci),'--'+str(dx),'--'+str(dy),'--'+str(dz), '--'+dirpath]
					subprocess.Popen(args)
				except StandardError:
					try:
						bexec = Blender.Get('homedir')
						bexec = bexec[:-8]
						bexec += 'blender'
						args = [bexec,'-b',path,'-P',dirpath+'../../source/blender/sphysics_export.py','--'+str(i),'--'+str(nproci),'--'+str(dx),'--'+str(dy),'--'+str(dz), '--'+dirpath]
						subprocess.Popen(args)
					except StandardError:
						print('Could not find Blender executable.')
						print('Please make a symlink to your blender executable in /usr/bin')
						print('STOP')
						return

# END MAIN()

main()
